"""jaseci URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.0/topics/http/urls/
Examples
    Function views
        1. Add an import:  from my_app import views
        2. Add a URL to urlpatterns:  path('', views.home, name='home')
    Class-based views
        1. Add an import:  from other_app.views import Home
        2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
    Including another URLconf
        1. Import the include() function: from django.urls import include, path
        2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import path, include
from rest_framework import permissions
from drf_yasg.views import get_schema_view
from drf_yasg import openapi


schema_view = get_schema_view(
    openapi.Info(
        title="Jaseci API",
        default_version='v1',
        description="Welcome to the world of Jaseci",
        terms_of_service="https://www.jaseci.org",
        contact=openapi.Contact(email="jason@jaseci.org"),
        license=openapi.License(name="Awesome Technology"),
    ),
    public=True,
    permission_classes=(permissions.AllowAny,),
)

urlpatterns = [
    path('docs/', schema_view.with_ui('swagger', cache_timeout=0),
         name='schema-swagger-ui'),
    path('redocs/', schema_view.with_ui('redoc', cache_timeout=0),
         name='schema-redoc'),
    path('admin/', admin.site.urls),
    path('user/', include('jaseci_serv.user_api.urls')),
    path('', include('jaseci_serv.jac_api.urls')),
    path('', include('jaseci_serv.obj_api.urls')),
]



from django.contrib.auth import get_user_model
from django.urls import reverse

from jaseci.utils.utils import TestCaseHelper
from django.test import TestCase

from rest_framework import status
from rest_framework.test import APIClient

from jaseci_serv.base.models import JaseciObject, GlobalVars


NODE_URL = reverse('obj_api:jaseciobject-list')
CONFIG_URL = reverse('obj_api:globalvars-list')


class PublicNodeApiTests(TestCaseHelper, TestCase):
    """Test the publicly available node API"""

    def setUp(self):
        super().setUp()
        self.client = APIClient()

    def tearDown(self):
        super().tearDown()

    def test_login_required_obj_api(self):
        """Test that login required for retrieving nodes"""
        res = self.client.get(NODE_URL)

        self.assertEqual(res.status_code, status.HTTP_401_UNAUTHORIZED)


class PrivateNodeApiTests(TestCaseHelper, TestCase):
    """Test the authorized user node API"""

    def setUp(self):
        super().setUp()
        self.user = get_user_model().objects.create_user(
            'JSCITfdfdEST_test@jaseci.com',
            'password'
        )
        self.client = APIClient()
        self.client.force_authenticate(self.user)

    def tearDown(self):
        super().tearDown()

    def test_retrieve_nodes(self):
        """Test retrieving node list"""
        JaseciObject.objects.create(name='Vegan')
        JaseciObject.objects.create(name='Dessert')

        res = self.client.get(NODE_URL)

        self.assertEqual(res.status_code, status.HTTP_200_OK)

    # def test_nodes_limited_to_user(self):
    #     """Test that nodes returned are for authenticated user"""
    #     user2 = get_user_model().objects.create_user(
    #         'JSCITEST_other@jaseci.com',
    #         'testpass'
    #     )
    #     JaseciObject.objects.create(name='Fruity')
    #     node = JaseciObject.objects.create(name='Comfort Food')

    #     res = self.client.get(NODE_URL)

    #     self.assertEqual(res.status_code, status.HTTP_200_OK)
    #     self.assertEqual(len(res.data['results']), 2)
    #     self.assertEqual(res.data['results'][1]['name'], node.name)
    #     user2.delete()

    def test_create_node_successful(self):
        """Test creating a new node"""
        payload = {'name': 'Simple', 'j_type': 'node'}
        self.client.post(NODE_URL, payload)

        exists = JaseciObject.objects.filter(
            name=payload['name']
        ).exists()
        self.assertTrue(exists)


class ConfigApiTests(TestCaseHelper, TestCase):
    """Test the authorized user node API"""

    def setUp(self):
        super().setUp()
        self.user = get_user_model().objects.create_superuser(
            'JSCITfdfdEST_test@jaseci.com',
            'password'
        )
        self.client = APIClient()
        self.client.force_authenticate(self.user)

    def tearDown(self):
        super().tearDown()

    def test_retrieve_configs(self):
        """Test retrieving config list"""
        GlobalVars.objects.create(name='Vegan', value='46')
        GlobalVars.objects.create(name='Dessert', value='446')

        res = self.client.get(CONFIG_URL)

        self.assertEqual(res.status_code, status.HTTP_200_OK)

    def test_create_config_successful(self):
        """Test creating a new config"""
        payload = {'name': 'Simple', 'value': 'Peezzy'}
        self.client.post(CONFIG_URL, payload)

        exists = GlobalVars.objects.filter(
            name=payload['name'],
            value=payload['value']
        ).exists()
        self.assertTrue(exists)

    def test_configs_limited_to_admin_user(self):
        """Test that configs only available to admin user"""
        user2 = get_user_model().objects.create_user(
            'JSCITEST_other@jaseci.com',
            'testpass'
        )
        client = APIClient()
        client.force_authenticate(user2)

        payload = {'name': 'Simple', 'value': 'Peezzy'}
        res = client.post(CONFIG_URL, payload)

        self.assertEqual(res.status_code, status.HTTP_403_FORBIDDEN)



from django.urls import path, include
from rest_framework.routers import DefaultRouter

from jaseci_serv.obj_api import views

router = DefaultRouter()
router.register('obj', views.ObjectViewSet)
router.register('global', views.ConfigViewSet)

app_name = 'obj_api'

urlpatterns = [
    path('', include(router.urls))
]



"""
Django settings for jaseci project.

Generated by 'django-admin startproject' using Django 3.0.3.

For more information on this file, see
https://docs.djangoproject.com/en/3.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.0/ref/settings/
"""

import os
import sys

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
TEMPLATE_DIR = os.path.join(BASE_DIR, 'ui/templates')

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '@%ad_kyis62uf7qf^w9kv(8$db4)%c$nnnjk^us=s@-gj*)aal'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'rest_framework',
    'knox',
    'django_rest_passwordreset',
    'drf_yasg',
    'jaseci_serv.base',
    'jaseci_serv.user_api',
    'jaseci_serv.obj_api',
    'jaseci_serv.jac_api',
    'corsheaders',
]

MIDDLEWARE = [
    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

CORS_ORIGIN_ALLOW_ALL = True

ROOT_URLCONF = 'jaseci_serv.jaseci_serv.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [TEMPLATE_DIR],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'jaseci_serv.jaseci_serv.wsgi.application'


# Database
# https://docs.djangoproject.com/en/3.0/ref/settings/#databases

if 'RDS_HOSTNAME' in os.environ:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.mysql',
            'NAME': os.environ['RDS_DB_NAME'],
            'USER': os.environ['RDS_USERNAME'],
            'PASSWORD': os.environ['RDS_PASSWORD'],
            'HOST': os.environ['RDS_HOSTNAME'],
            'PORT': os.environ['RDS_PORT'],
        }
    }

elif 'POSTGRES_HOST' in os.environ:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql',
            'HOST': os.environ.get('POSTGRES_HOST'),
            'NAME': os.environ.get('POSTGRES_USER'),
            'USER': os.environ.get('POSTGRES_USER'),
            'PASSWORD': os.environ.get('POSTGRES_PASSWORD'),
            'PORT': os.getenv('POSTGRES_PORT', 5432),
        }
    }

else:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': 'mydatabase',
        }
    }

# Cover regular testing and django-coverage
if 'test' in sys.argv or 'test_coverage' in sys.argv:
    DATABASES['default']['ENGINE'] = 'django.db.backends.sqlite3'
    DATABASES['default']['NAME'] = ':memory:'

# REDIS
REDIS_HOST = os.getenv('REDIS_HOST', 'localhost')
REDIS_PORT = os.getenv('REDIS_PORT', '6379')
REDIS_URL = "redis://{host}:{port}/1".format(
    host=REDIS_HOST,
    port=REDIS_PORT
)

# Password validation
# https://docs.djangoproject.com/en/3.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.' +
                'UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.' +
                'MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.' +
                'CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.' +
                'NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/3.0/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = False


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.0/howto/static-files/

STATIC_URL = '/static/'
STATICFILES_DIRS = [os.path.join(BASE_DIR, 'static')]

# Sets global default user model to be custom
AUTH_USER_MODEL = 'base.User'

REST_FRAMEWORK = {
    'DEFAULT_PAGINATION_CLASS':
    'rest_framework.pagination.PageNumberPagination',
    'PAGE_SIZE': 100,
    'DEFAULT_AUTHENTICATION_CLASSES': [
        'knox.auth.TokenAuthentication',
    ]
}

REST_KNOX = {
    'TOKEN_TTL': None,
}

LOGIN_URL = '/ui/login'
LOGIN_REDIRECT_URL = '/ui'
LOGOUT_REDIRECT_URL = '/ui'

# Configs to Manage by Jaseci
JASECI_CONFIGS = ['EMAIL_BACKEND',
                  'EMAIL_USE_TLS',
                  'EMAIL_HOST',
                  'EMAIL_HOST_USER',
                  'EMAIL_HOST_PASSWORD',
                  'EMAIL_PORT',
                  'EMAIL_DEFAULT_FROM',
                  'EMAIL_ACTIVATION_SUBJ',
                  'EMAIL_ACTIVATION_BODY',
                  'EMAIL_ACTIVATION_HTML_BODY',
                  'EMAIL_RESETPASS_SUBJ',
                  'EMAIL_RESETPASS_BODY',
                  'EMAIL_RESETPASS_HTML_BODY',
                  'LOGIN_TOKEN_TTL_HOURS'
                  ]



import uuid
from datetime import datetime

from django.db import models
from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, \
    PermissionsMixin
from jaseci_serv.jaseci_serv.settings import JASECI_CONFIGS
from django.contrib.auth import get_user_model
from jaseci_serv.base.orm_hook import orm_hook
from jaseci.element.master import master as core_master
from jaseci.element.super_master import super_master as core_super
from jaseci.api.interface import interface


class master(core_master):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._valid_configs += JASECI_CONFIGS

    @interface.private_api()
    def master_create(self, name: str, set_active: bool = True,
                      other_fields: dict = {}):
        """
        Create a master instance and return root node master object

        other_fields used for additional fields for overloaded interfaces
        (i.e., Django interface)
        """
        data = {'email': name}
        for i in other_fields.keys():
            data[i] = other_fields[i]
        from jaseci_serv.user_api.serializers import UserSerializer
        serializer = UserSerializer(data=data)
        if(serializer.is_valid(raise_exception=False)):
            mas = serializer.save().get_master()
            mas._h = self._h
            return self.make_me_head_master_or_destroy(mas)
        else:
            return {'response': "Errors occurred",
                    'errors': serializer.errors}

    @interface.private_api()
    def master_delete(self, name: str):
        """
        Permanently delete master with given id
        """
        if(not self.sub_master_ids.has_obj_by_name(name)):
            return {'response': f"{name} not found"}
        self.sub_master_ids.destroy_obj_by_name(name)
        get_user_model().objects.get(email=name).delete()
        return {'response': f"{name} has been destroyed"}


class super_master(master, core_super):

    @interface.admin_api()
    def master_createsuper(self, name: str, set_active: bool = True,
                           other_fields: dict = {}):
        """
        Create a super instance and return root node super object
        """
        data = {'email': name}
        for i in other_fields.keys():
            data[i] = other_fields[i]
        from jaseci_serv.user_api.serializers import SuperUserSerializer
        serializer = SuperUserSerializer(data=data)
        if(serializer.is_valid(raise_exception=False)):
            mas = serializer.save().get_master()
            mas._h = self._h
            return self.make_me_head_master_or_destroy(mas)
        else:
            return {'response': "Errors occurred",
                    'errors': serializer.errors}

    @interface.admin_api()
    def master_allusers(self, num: int = 0, start_idx: int = 0):
        """
        Returns info on a set of users, num specifies the number of users to
        return and start idx specfies where to start
        """
        users = get_user_model().objects.all()
        start = start_idx if start_idx else 0
        end = start_idx + num if num else len(users)
        ret = []
        for i in users[start:end]:
            ret.append({'user': i.email, 'jid': i.master.urn})
        return ret


class UserManager(BaseUserManager):
    """
    Custom User Manager for Jaseci

    Note: Every user is linked to a single root node that is created upon the
    creation of the user.
    """

    def create_user(self, email, password=None, **extra_fields):
        """Creates and saves a new user"""
        # Makes first user admin
        if(not get_user_model().objects.filter(is_admin=True).exists()):
            return self.create_superuser(email, password, **extra_fields)
        email = self.normalize_email(email).lower()
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)

        # Create user's root node
        user.master = master(h=user._h, name=email).id
        if('set_sent_global' in extra_fields and
           extra_fields['set_sent_global']):
            user.master.sentinel_active_global()
        if('create_graph' in extra_fields and
           extra_fields['create_graph']):
            user.master.graph_create()
        user._h.commit()

        user.save(using=self._db)
        return user

    def create_superuser(self, email, password=None, **extra_fields):
        """Creates and saves a new super user"""
        email = self.normalize_email(email).lower()
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.is_staff = True
        user.is_activated = True
        user.is_admin = True
        user.is_superuser = True
        user.save(using=self._db)

        # Create user's root node
        user.master = super_master(h=user._h, name=email).id
        user._h.commit()

        user.save(using=self._db)
        return user


class User(AbstractBaseUser, PermissionsMixin):
    """
    Custom User Model to use email instead of username

    Root node  is attached to each User and created at user creation
    """
    email = models.EmailField(max_length=255, unique=True)
    name = models.CharField(max_length=255, blank=True)
    time_created = models.DateTimeField(auto_now_add=True)
    time_modified = models.DateTimeField(auto_now=True)
    is_active = models.BooleanField(default=True)
    is_activated = models.BooleanField(default=False)
    is_staff = models.BooleanField(default=False)
    is_admin = models.BooleanField(default=False)
    is_superuser = models.BooleanField(default=False)
    master = models.UUIDField(default=uuid.uuid4)
    objects = UserManager()

    def __init__(self, *args, **kwargs):
        self._h = orm_hook(
            objects=JaseciObject.objects,
            globs=GlobalVars.objects
        )
        AbstractBaseUser.__init__(self, *args, **kwargs)
        PermissionsMixin.__init__(self, *args, **kwargs)

    USERNAME_FIELD = 'email'

    def get_master(self):
        """Returns main user Jaseci node"""
        return self._h.get_obj(caller_id=self.master.urn, item_id=self.master)

    def delete(self):
        JaseciObject.objects.filter(j_master=self.master.urn).delete()
        JaseciObject.objects.filter(jid=self.master.urn).delete()
        super().delete()


class JaseciObject(models.Model):
    """
    Generalized object model for Jaseci object types

    There is one table in db for all object types in the Jaseci machine
    which include nodes, edges, actions, contexts, walkers, and
    sentinels.

    Keep in mind Django's ORM can support many to many relationships
    between recursive schema's which may be useful if necessary in the
    future.
    """
    jid = models.UUIDField(
        primary_key=True, default=uuid.uuid4, editable=False)
    j_parent = models.UUIDField(null=True, blank=True)
    j_master = models.UUIDField(null=True, blank=True)
    name = models.CharField(max_length=255, blank=True)
    kind = models.CharField(max_length=255, blank=True)
    j_timestamp = models.DateTimeField(default=datetime.utcnow)
    j_type = models.CharField(max_length=15, default='node')
    j_access = models.CharField(max_length=15, default='private')
    j_r_acc_ids = models.TextField(blank=True)
    j_rw_acc_ids = models.TextField(blank=True)
    jsci_obj = models.TextField(blank=True)


class GlobalVars(models.Model):
    """Global configuration item"""
    name = models.CharField(max_length=31, unique=True)
    value = models.TextField(blank=True)


def lookup_global_config(name, default=None):
    """Helper for looking up GlobalVars, returns default if not found"""
    try:
        return GlobalVars.objects.get(name=name).value
    except GlobalVars.DoesNotExist:
        return default



from django.contrib.auth import get_user_model

from jaseci.utils.utils import TestCaseHelper
from django.test import TestCase

from jaseci_serv.base.models import GlobalVars
from jaseci.utils.mem_hook import mem_hook

# Alias for create user
create_user = get_user_model().objects.create_user
get_user = get_user_model().objects.get


class jaseci_engine_orm_config_tests_private(TestCaseHelper, TestCase):
    """Test Jaseci Engine when authenticated"""

    def setUp(self):
        super().setUp()
        self.user = create_user(
            email='JSCadft@jaseci.com',
            password='testpass',
            name='some dude',
        )
        self._h = mem_hook()

    def tearDown(self):
        super().tearDown()

    def test_jsci_db_to_engine_hook_saving(self):
        """Test that db hooks are set up correctly for saving"""
        user = self.user
        self.assertIsNotNone(user._h)
        h = user._h
        h.save_glob('GOOBY', 'MOOBY')
        h.commit()

        load_test = GlobalVars.objects.get(name='GOOBY')
        self.assertEqual(load_test.value, 'MOOBY')

    def test_jsci_db_to_engine_hook_loading(self):
        """Test that db glob hooks are set up correctly for loading"""
        user = self.user
        self.assertIsNotNone(user._h)
        h = user._h
        h.save_glob('GOOBY1', 'MOOBY1')
        h.commit()

        del user._h.mem['global']['GOOBY1']
        user._h.red.delete('GOOBY1')
        self.assertNotIn('GOOBY1', user._h.mem['global'].keys())

        load_test = GlobalVars.objects.filter(name='GOOBY1').first()
        self.assertIsInstance(load_test, GlobalVars)

        user._h.get_glob('GOOBY1')
        self.assertEqual(user._h.mem['global']['GOOBY1'], 'MOOBY1')

    def test_jsci_db_glob_listing(self):
        """Test that db glob hooks are set up correctly for listing"""
        user = self.user
        self.assertIsNotNone(user._h)
        h = user._h
        h.save_glob('GOOBY1', 'MOOBY1')
        h.save_glob('GOOBY2', 'MOOBY2')
        h.save_glob('GOOBY3', 'MOOBY3')
        h.commit()

        user._h.clear_mem_cache()
        user._h.red.delete('GOOBY1')
        self.assertEqual(len(user._h.mem['global'].keys()), 0)
        self.assertNotIn('GOOBY1', user._h.mem['global'].keys())

        li = user._h.list_glob()
        self.assertEqual(len(li), 3)
        self.assertEqual(len(user._h.mem['global'].keys()), 0)
        user._h.get_glob('GOOBY1')
        self.assertEqual(user._h.mem['global']['GOOBY1'], 'MOOBY1')



from django.contrib.auth import get_user_model
from django.urls import reverse

from rest_framework.test import APIClient
import jaseci.actions.live_actions as lact
from jaseci.utils.utils import TestCaseHelper
from django.test import TestCase
import uuid
import base64
import os


class test_zsb(TestCaseHelper, TestCase):
    """Test the authorized user node API"""

    def setUp(self):
        super().setUp()
        self.user = get_user_model().objects.create_user(
            'JSCITfdfdEST_test@jaseci.com',
            'password'
        )
        self.client = APIClient()
        self.client.force_authenticate(self.user)
        self.master = self.user.get_master()
        zsb_loc = os.path.dirname(__file__) + '/zsb.jac'
        ll_file = base64.b64encode(
            open(zsb_loc).read().encode()).decode()
        payload = {'op': 'sentinel_register',
                   'name': 'Something', 'code': ll_file, 'encoded': True}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.snt = self.master._h.get_obj(
            self.master.jid, uuid.UUID(res.data[0]['jid']))
        self.gph = self.master._h.get_obj(
            self.master.jid, uuid.UUID(res.data[1]['jid']))

    def tearDown(self):
        super().tearDown()

    def run_walker(self, w_name, ctx, prime=None):
        """Helper to make calls to execute walkers"""
        if(not prime):
            payload = {'snt': self.snt.id.urn, 'name': w_name,
                       'nd': self.gph.id.urn, 'ctx': ctx}
        else:
            payload = {'snt': self.snt.id.urn, 'name': w_name,
                       'nd': prime, 'ctx': ctx}
        res = self.client.post(
            reverse('jac_api:walker_run'), payload, format='json')
        return res.data

    def test_zsb_create_answer(self):
        """Test ZSB Create Answer call USE api"""
        if (not lact.load_remote_actions('http://js-use-qa')):
            self.skipTest("external resource not available")
        data = self.run_walker('add_bot', {'name': "Bot"})
        self.assertEqual(data['report'][0]['name'], 'bot')
        bot_jid = data['report'][0]['jid']
        data = self.run_walker('create_answer', {'text': "Yep"}, prime=bot_jid)
        self.assertEqual(data['report'][0]['name'], 'answer')

    def test_zsb_ask_question(self):
        """Test ZSB Create Answer call USE api"""
        if (not lact.load_remote_actions('http://js-use-qa')):
            self.skipTest("external resource not available")
        data = self.run_walker('add_bot', {'name': "Bot"})
        self.assertEqual(data['report'][0]['name'], 'bot')
        bot_jid = data['report'][0]['jid']
        data = self.run_walker('create_answer', {'text': "Yep"}, prime=bot_jid)
        data = self.run_walker(
            'create_answer', {'text': "Nope"}, prime=bot_jid)
        data = self.run_walker(
            'create_answer', {'text': "Maybe"}, prime=bot_jid)
        data = self.run_walker(
            'ask_question', {'text': "Who says yep?"}, prime=bot_jid)
        data = self.run_walker(
            'get_log', {}, prime=bot_jid)
        self.assertEqual(data['report'][0][0][1], 'Who says yep?')

    def test_zsb_ask_question_multi(self):
        """Test ZSB Create Answer call USE api"""
        if (not lact.load_remote_actions('http://js-use-qa')):
            self.skipTest("external resource not available")
        data = self.run_walker('add_bot', {'name': "Bot"})
        self.assertEqual(data['report'][0]['name'], 'bot')
        bot_jid = data['report'][0]['jid']
        data = self.run_walker('create_answer', {'text': "Yep"}, prime=bot_jid)
        data = self.run_walker(
            'create_answer', {'text': "Nope"}, prime=bot_jid)
        data = self.run_walker(
            'create_answer', {'text': "Maybe"}, prime=bot_jid)
        data = self.run_walker(
            'ask_question', {'text': "Who says yep?"}, prime=bot_jid)
        data = self.run_walker(
            'ask_question', {'text': "Who says yep?"}, prime=bot_jid)
        data = self.run_walker(
            'ask_question', {'text': "Who says yep?"}, prime=bot_jid)
        data = self.run_walker(
            'get_log', {}, prime=bot_jid)
        self.assertEqual(data['report'][0][0][1], 'Who says yep?')



from django.urls import path, include

from . import views

app_name = 'user_api'

urlpatterns = [
    path('create/', views.CreateUserView.as_view(), name='create'),
    path('create_super/', views.CreateSuperUserView.as_view(),
         name='create_super'),
    path('token/', views.CreateTokenView.as_view(), name='token'),
    path('manage/', views.ManageUserView.as_view(), name='manage'),
    path('logout_everyone/', views.LogoutAllUsersView.as_view(),
         name='logout_everyone'),
    path('activate/<str:code>', views.ActivateUserView.as_view(),
         name='activate'),
    path('password_reset/',
         include('django_rest_passwordreset.urls',
                 namespace='password_reset')),
]
